/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** File:		pralarm.h^M
** Description:	API to periodic alarms.^M
**^M
**^M
** Alarms are defined to invoke some client specified function at ^M
** a time in the future. The notification may be a one time event^M
** or repeated at a fixed interval. The interval at which the next^M
** notification takes place may be modified by the client code only^M
** during the respective notification.^M
**^M
** The notification is delivered on a thread that is part of the^M
** alarm context (PRAlarm). The thread will inherit the priority^M
** of the Alarm creator.^M
**^M
** Any number of periodic alarms (PRAlarmID) may be created within^M
** the context of a single alarm (PRAlarm). The notifications will be^M
** scheduled as close to the desired time as possible.^M
**^M
** Repeating periodic notifies are expected to run at a fixed rate.^M
** That rate is expressed as some number of notifies per period where^M
** the period is much larger than a PRIntervalTime (see prinrval.h).^M
*/^M
^M
#if !defined(pralarm_h)^M
#define pralarm_h^M
^M
#include "prtypes.h"^M
#include "prinrval.h"^M
^M
^M
PR_BEGIN_EXTERN_C^M
^M
/**********************************************************************/^M
/************************* TYPES AND CONSTANTS ************************/^M
/**********************************************************************/^M
^M
typedef struct PRAlarm PRAlarm;^M
typedef struct PRAlarmID PRAlarmID;^M
^M
typedef PRBool (PR_CALLBACK *PRPeriodicAlarmFn)(^M
    PRAlarmID *id, void *clientData, PRUint32 late);^M
^M
/**********************************************************************/^M
/****************************** FUNCTIONS *****************************/^M
/**********************************************************************/^M
^M
/***********************************************************************^M
** FUNCTION:    PR_CreateAlarm^M
** DESCRIPTION:^M
**  Create an alarm context.^M
** INPUTS:      void^M
** OUTPUTS:     None^M
** RETURN:      PRAlarm*^M
**  ^M
** SIDE EFFECTS:^M
**  This creates an alarm context, which is an object used for subsequent^M
**  notification creations. It also creates a thread that will be used to^M
** deliver the notifications that are expected to be defined. The client^M
** is resposible for destroying the context when appropriate.^M
** RESTRICTIONS:^M
**  None. ^M
** MEMORY:      The object (PRAlarm) and a thread to support notifications.^M
** ALGORITHM:   N/A^M
***********************************************************************/^M
NSPR_API(PRAlarm*) PR_CreateAlarm(void);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_DestroyAlarm^M
** DESCRIPTION:^M
**  Destroys the context created by PR_CreateAlarm().^M
** INPUTS:      PRAlarm*^M
** OUTPUTS:     None^M
** RETURN:      PRStatus^M
**  ^M
** SIDE EFFECTS:^M
**  This destroys the context that was created by PR_CreateAlarm().^M
**  If there are any active alarms (PRAlarmID), they will be cancelled.^M
**  Once that is done, the thread that was used to deliver the alarms^M
**  will be joined. ^M
** RESTRICTIONS:^M
**  None. ^M
** MEMORY:      N/A^M
** ALGORITHM:   N/A^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_DestroyAlarm(PRAlarm *alarm);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_SetAlarm^M
** DESCRIPTION:^M
**  Creates a periodic notifier that is to be delivered to a specified^M
**  function at some fixed interval.^M
** INPUTS:      PRAlarm *alarm              Parent alarm context^M
**              PRIntervalTime period       Interval over which the notifies^M
**                                          are delivered.^M
**              PRUint32 rate               The rate within the interval that^M
**                                          the notifies will be delivered.^M
**              PRPeriodicAlarmFn function  Entry point where the notifies^M
**                                          will be delivered.^M
** OUTPUTS:     None^M
** RETURN:      PRAlarmID*                  Handle to the notifier just created^M
**                                          or NULL if the request failed.^M
**  ^M
** SIDE EFFECTS:^M
**  A periodic notifier is created. The notifications will be delivered^M
**  by the alarm's internal thread at a fixed interval whose rate is the^M
**  number of interrupts per interval specified. The first notification^M
**  will be delivered as soon as possible, and they will continue until^M
**  the notifier routine indicates that they should cease of the alarm^M
**  context is destroyed (PR_DestroyAlarm).^M
** RESTRICTIONS:^M
**  None. ^M
** MEMORY:      Memory for the notifier object.^M
** ALGORITHM:   The rate at which notifications are delivered are stated^M
**              to be "'rate' notifies per 'interval'". The exact time of^M
**              the notification is computed based on a epoch established^M
**              when the notifier was set. Each notification is delivered^M
**              not ealier than the epoch plus the fixed rate times the^M
**              notification sequence number. Such notifications have the^M
**              potential to be late by not more than 'interval'/'rate'.^M
**              The amount of lateness of one notification is taken into^M
**              account on the next in an attempt to avoid long term slew.  ^M
***********************************************************************/^M
NSPR_API(PRAlarmID*) PR_SetAlarm(^M
    PRAlarm *alarm, PRIntervalTime period, PRUint32 rate,^M
    PRPeriodicAlarmFn function, void *clientData);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_ResetAlarm^M
** DESCRIPTION:^M
**  Resets an existing alarm.^M
** INPUTS:      PRAlarmID *id               Identify of the notifier.^M
**              PRIntervalTime period       Interval over which the notifies^M
**                                          are delivered.^M
**              PRUint32 rate               The rate within the interval that^M
**                                          the notifies will be delivered.^M
** OUTPUTS:     None^M
** RETURN:      PRStatus                    Indication of completion.^M
**  ^M
** SIDE EFFECTS:^M
**  An existing alarm may have its period and rate redefined. The^M
**  additional side effect is that the notifier's epoch is recomputed.^M
**  The first notification delivered by the newly refreshed alarm is^M
**  defined to be 'interval'/'rate' from the time of the reset.^M
** RESTRICTIONS:^M
**  This function may only be called in the notifier for that alarm.^M
** MEMORY:      N/A.^M
** ALGORITHM:   See PR_SetAlarm().  ^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_ResetAlarm(^M
	PRAlarmID *id, PRIntervalTime period, PRUint32 rate);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* !defined(pralarm_h) */^M
^M
/* prinrval.h */^M
