/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prcvar_h___^M
#define prcvar_h___^M
^M
#include "prlock.h"^M
#include "prinrval.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
typedef struct PRCondVar PRCondVar;^M
^M
/*^M
** Create a new condition variable.^M
**^M
** 	"lock" is the lock used to protect the condition variable.^M
**^M
** Condition variables are synchronization objects that threads can use^M
** to wait for some condition to occur.^M
**^M
** This may fail if memory is tight or if some operating system resource^M
** is low. In such cases, a NULL will be returned.^M
*/^M
NSPR_API(PRCondVar*) PR_NewCondVar(PRLock *lock);^M
^M
/*^M
** Destroy a condition variable. There must be no thread^M
** waiting on the condvar. The caller is responsible for guaranteeing^M
** that the condvar is no longer in use.^M
**^M
*/^M
NSPR_API(void) PR_DestroyCondVar(PRCondVar *cvar);^M
^M
/*^M
** The thread that waits on a condition is blocked in a "waiting on^M
** condition" state until another thread notifies the condition or a^M
** caller specified amount of time expires. The lock associated with^M
** the condition variable will be released, which must have be held^M
** prior to the call to wait.^M
**^M
** Logically a notified thread is moved from the "waiting on condition"^M
** state and made "ready." When scheduled, it will attempt to reacquire^M
** the lock that it held when wait was called.^M
**^M
** The timeout has two well known values, PR_INTERVAL_NO_TIMEOUT and^M
** PR_INTERVAL_NO_WAIT. The former value requires that a condition be^M
** notified (or the thread interrupted) before it will resume from the^M
** wait. If the timeout has a value of PR_INTERVAL_NO_WAIT, the effect^M
** is to release the lock, possibly causing a rescheduling within the^M
** runtime, then immediately attempting to reacquire the lock and resume.^M
**^M
** Any other value for timeout will cause the thread to be rescheduled^M
** either due to explicit notification or an expired interval. The latter^M
** must be determined by treating time as one part of the monitored data^M
** being protected by the lock and tested explicitly for an expired^M
** interval.^M
**^M
** Returns PR_FAILURE if the caller has not locked the lock associated^M
** with the condition variable or the thread was interrupted (PR_Interrupt()).^M
** The particular reason can be extracted with PR_GetError().^M
*/^M
NSPR_API(PRStatus) PR_WaitCondVar(PRCondVar *cvar, PRIntervalTime timeout);^M
^M
/*^M
** Notify ONE thread that is currently waiting on 'cvar'. Which thread is^M
** dependent on the implementation of the runtime. Common sense would dictate^M
** that all threads waiting on a single condition have identical semantics,^M
** therefore which one gets notified is not significant. ^M
**^M
** The calling thead must hold the lock that protects the condition, as^M
** well as the invariants that are tightly bound to the condition, when^M
** notify is called.^M
**^M
** Returns PR_FAILURE if the caller has not locked the lock associated^M
** with the condition variable.^M
*/^M
NSPR_API(PRStatus) PR_NotifyCondVar(PRCondVar *cvar);^M
^M
/*^M
** Notify all of the threads waiting on the condition variable. The order^M
** that the threads are notified is indeterminant. The lock that protects^M
** the condition must be held.^M
**^M
** Returns PR_FAILURE if the caller has not locked the lock associated^M
** with the condition variable.^M
*/^M
NSPR_API(PRStatus) PR_NotifyAllCondVar(PRCondVar *cvar);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prcvar_h___ */^M
