/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** File: prmem.h^M
** Description: API to NSPR 2.0 memory management functions^M
**^M
*/^M
#ifndef prmem_h___^M
#define prmem_h___^M
^M
#include "prtypes.h"^M
#include <stddef.h>^M
#include <stdlib.h>^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** Thread safe memory allocation.^M
**^M
** NOTE: pr wraps up malloc, free, calloc, realloc so they are already^M
** thread safe (and are not declared here - look in stdlib.h).^M
*/^M
^M
/*^M
** PR_Malloc, PR_Calloc, PR_Realloc, and PR_Free have the same signatures^M
** as their libc equivalent malloc, calloc, realloc, and free, and have^M
** the same semantics.  (Note that the argument type size_t is replaced^M
** by PRUint32.)  Memory allocated by PR_Malloc, PR_Calloc, or PR_Realloc^M
** must be freed by PR_Free.^M
*/^M
^M
NSPR_API(void *) PR_Malloc(PRUint32 size);^M
^M
NSPR_API(void *) PR_Calloc(PRUint32 nelem, PRUint32 elsize);^M
^M
NSPR_API(void *) PR_Realloc(void *ptr, PRUint32 size);^M
^M
NSPR_API(void) PR_Free(void *ptr);^M
^M
/*^M
** The following are some convenience macros defined in terms of^M
** PR_Malloc, PR_Calloc, PR_Realloc, and PR_Free.^M
*/^M
^M
/***********************************************************************^M
** FUNCTION:	PR_MALLOC()^M
** DESCRIPTION:^M
**   PR_NEW() allocates an untyped item of size _size from the heap.^M
** INPUTS:  _size: size in bytes of item to be allocated^M
** OUTPUTS:	untyped pointer to the node allocated^M
** RETURN:	pointer to node or error returned from malloc().^M
***********************************************************************/^M
#define PR_MALLOC(_bytes) (PR_Malloc((_bytes)))^M
^M
/***********************************************************************^M
** FUNCTION:	PR_NEW()^M
** DESCRIPTION:^M
**   PR_NEW() allocates an item of type _struct from the heap.^M
** INPUTS:  _struct: a data type^M
** OUTPUTS:	pointer to _struct^M
** RETURN:	pointer to _struct or error returns from malloc().^M
***********************************************************************/^M
#define PR_NEW(_struct) ((_struct *) PR_MALLOC(sizeof(_struct)))^M
^M
/***********************************************************************^M
** FUNCTION:	PR_REALLOC()^M
** DESCRIPTION:^M
**   PR_REALLOC() re-allocates _ptr bytes from the heap as a _size^M
**   untyped item.^M
** INPUTS:	_ptr: pointer to node to reallocate^M
**          _size: size of node to allocate^M
** OUTPUTS:	pointer to node allocated^M
** RETURN:	pointer to node allocated^M
***********************************************************************/^M
#define PR_REALLOC(_ptr, _size) (PR_Realloc((_ptr), (_size)))^M
^M
/***********************************************************************^M
** FUNCTION:	PR_CALLOC()^M
** DESCRIPTION:^M
**   PR_CALLOC() allocates a _size bytes untyped item from the heap^M
**   and sets the allocated memory to all 0x00.^M
** INPUTS:	_size: size of node to allocate^M
** OUTPUTS:	pointer to node allocated^M
** RETURN:	pointer to node allocated^M
***********************************************************************/^M
#define PR_CALLOC(_size) (PR_Calloc(1, (_size)))^M
^M
/***********************************************************************^M
** FUNCTION:	PR_NEWZAP()^M
** DESCRIPTION:^M
**   PR_NEWZAP() allocates an item of type _struct from the heap^M
**   and sets the allocated memory to all 0x00.^M
** INPUTS:	_struct: a data type^M
** OUTPUTS:	pointer to _struct^M
** RETURN:	pointer to _struct^M
***********************************************************************/^M
#define PR_NEWZAP(_struct) ((_struct*)PR_Calloc(1, sizeof(_struct)))^M
^M
/***********************************************************************^M
** FUNCTION:	PR_DELETE()^M
** DESCRIPTION:^M
**   PR_DELETE() unallocates an object previosly allocated via PR_NEW()^M
**   or PR_NEWZAP() to the heap.^M
** INPUTS:	pointer to previously allocated object^M
** OUTPUTS:	the referenced object is returned to the heap^M
** RETURN:	void^M
***********************************************************************/^M
#define PR_DELETE(_ptr) { PR_Free(_ptr); (_ptr) = NULL; }^M
^M
/***********************************************************************^M
** FUNCTION:	PR_FREEIF()^M
** DESCRIPTION:^M
**   PR_FREEIF() conditionally unallocates an object previously allocated^M
**   vial PR_NEW() or PR_NEWZAP(). If the pointer to the object is^M
**   equal to zero (0), the object is not released.^M
** INPUTS:	pointer to previously allocated object^M
** OUTPUTS:	the referenced object is conditionally returned to the heap^M
** RETURN:	void^M
***********************************************************************/^M
#define PR_FREEIF(_ptr)	if (_ptr) PR_DELETE(_ptr)^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prmem_h___ */^M
