/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions requiored by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prolock_h___^M
#define prolock_h___^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
#if defined(DEBUG) || defined(FORCE_NSPR_ORDERED_LOCKS)^M
^M
/*^M
** A locking mechanism, built on the existing PRLock definiion,^M
** is provided that will permit applications to define a Lock^M
** Hierarchy (or Lock Ordering) schema. An application designed^M
** using the Ordered Lock functions will terminate with a^M
** diagnostic message when a lock inversion condition is^M
** detected. ^M
** ^M
** The lock ordering detection is complile-time enabled only. in^M
** optimized builds of NSPR, the Ordered Lock functions map^M
** directly to PRLock functions, providing no lock order^M
** detection.^M
** ^M
** The Ordered Lock Facility is compiled in when DEBUG is defined at^M
** compile time. Ordered Lock can be forced on in optimized builds by^M
** defining FORCE_NSPR_ORDERED_LOCK at compile time. Both the^M
** application using Ordered Lock and NSPR must be compiled with the^M
** facility enabled to achieve the desired results.^M
** ^M
** Application designers should use the macro interfaces to the Ordered^M
** Lock facility to ensure that it is compiled out in optimized builds.^M
**^M
** Application designers are responsible for defining their own^M
** lock hierarchy. ^M
**^M
** Ordered Lock is thread-safe and SMP safe.^M
**^M
** See Also: prlock.h^M
**^M
** /lth. 10-Jun-1998.^M
**^M
*/^M
^M
/*^M
** Opaque type for ordered lock.^M
** ... Don't even think of looking in here.^M
**^M
*/^M
^M
typedef void * PROrderedLock;^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_CreateOrderedLock() -- Create an Ordered Lock^M
** ^M
** DESCRIPTION: PR_CreateOrderedLock() creates an ordered lock.^M
** ^M
** INPUTS:^M
**  order: user defined order of this lock.^M
**  name: name of the lock. For debugging purposes.^M
** ^M
** OUTPUTS: returned^M
** ^M
** RETURNS: PR_OrderedLock pointer^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_CREATE_ORDERED_LOCK(order,name)\^M
    PR_CreateOrderedLock((order),(name))^M
^M
NSPR_API(PROrderedLock *) ^M
    PR_CreateOrderedLock( ^M
        PRInt32 order,^M
        const char *name^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_DestroyOrderedLock() -- Destroy an Ordered Lock^M
** ^M
** DESCRIPTION: PR_DestroyOrderedLock() destroys the ordered lock^M
** referenced by lock.^M
** ^M
** INPUTS: lock: pointer to a PROrderedLock^M
** ^M
** OUTPUTS: the lock is destroyed^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_DESTROY_ORDERED_LOCK(lock) PR_DestroyOrderedLock((lock))^M
^M
NSPR_API(void) ^M
    PR_DestroyOrderedLock( ^M
        PROrderedLock *lock ^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_LockOrderedLock() -- Lock an ordered lock^M
** ^M
** DESCRIPTION: PR_LockOrderedLock() locks the ordered lock^M
** referenced by lock. If the order of lock is less than or equal^M
** to the order of the highest lock held by the locking thread,^M
** the function asserts.^M
** ^M
** INPUTS: lock: a pointer to a PROrderedLock^M
** ^M
** OUTPUTS: The lock is held or the fucntion asserts.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_LOCK_ORDERED_LOCK(lock) PR_LockOrderedLock((lock))^M
^M
NSPR_API(void) ^M
    PR_LockOrderedLock( ^M
        PROrderedLock *lock ^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_UnlockOrderedLock() -- unlock and Ordered Lock^M
** ^M
** DESCRIPTION: PR_UnlockOrderedLock() unlocks the lock referenced^M
** by lock.^M
** ^M
** INPUTS: lock: a pointer to a PROrderedLock^M
** ^M
** OUTPUTS: the lock is unlocked^M
** ^M
** RETURNS:^M
**  PR_SUCCESS^M
**  PR_FAILURE^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_UNLOCK_ORDERED_LOCK(lock) PR_UnlockOrderedLock((lock))^M
^M
NSPR_API(PRStatus) ^M
    PR_UnlockOrderedLock( ^M
        PROrderedLock *lock ^M
);^M
^M
#else /* !(defined(DEBUG) || defined(FORCE_NSPR_ORDERED_LOCKS)) */^M
/*^M
** Map PROrderedLock and methods onto PRLock when ordered locking^M
** is not compiled in.^M
**  ^M
*/^M
#include <prlock.h>^M
^M
typedef PRLock PROrderedLock;^M
^M
#define PR_CREATE_ORDERED_LOCK(order) PR_NewLock()^M
#define PR_DESTROY_ORDERED_LOCK(lock) PR_DestroyLock((lock))^M
#define PR_LOCK_ORDERED_LOCK(lock) PR_Lock((lock))^M
#define PR_UNLOCK_ORDERED_LOCK(lock) PR_Unlock((lock))^M
^M
#endif /* !(defined(DEBUG) || defined(FORCE_NSPR_ORDERED_LOCKS)) */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prolock_h___ */^M
^M
^M
^M
^M
^M
^M
^M
^M
^M
^M
^M
^M
^M
