/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1999-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** prshma.h -- NSPR Anonymous Shared Memory^M
**^M
** NSPR provides an anonymous shared memory based on NSPR's PRFileMap^M
** type. The anonymous file-mapped shared memory provides an inheritable^M
** shared memory, as in: the child process inherits the shared memory.^M
** Compare the file-mapped anonymous shared memory to to a named shared^M
** memory described in prshm.h. The intent is to provide a shared^M
** memory that is accessable only by parent and child processes. ...^M
** It's a security thing.^M
** ^M
** Depending on the underlying platform, the file-mapped shared memory^M
** may be backed by a file. ... surprise! ... On some platforms, no^M
** real file backs the shared memory. On platforms where the shared^M
** memory is backed by a file, the file's name in the filesystem is^M
** visible to other processes for only the duration of the creation of^M
** the file, hopefully a very short time. This restricts processess^M
** that do not inherit the shared memory from opening the file and^M
** reading or writing its contents. Further, when all processes^M
** using an anonymous shared memory terminate, the backing file is^M
** deleted. ... If you are not paranoid, you're not paying attention.^M
** ^M
** The file-mapped shared memory requires a protocol for the parent^M
** process and child process to share the memory. NSPR provides two^M
** protocols. Use one or the other; don't mix and match.^M
** ^M
** In the first protocol, the job of passing the inheritable shared^M
** memory is done via helper-functions with PR_CreateProcess(). In the^M
** second protocol, the parent process is responsible for creating the^M
** child process; the parent and child are mutually responsible for^M
** passing a FileMap string. NSPR provides helper functions for^M
** extracting data from the PRFileMap object. ... See the examples^M
** below.^M
** ^M
** Both sides should adhere strictly to the protocol for proper^M
** operation. The pseudo-code below shows the use of a file-mapped^M
** shared memory by a parent and child processes. In the examples, the^M
** server creates the file-mapped shared memory, the client attaches to^M
** it.^M
**^M
** First protocol.^M
** Server:^M
**^M
**   fm = PR_OpenAnonFileMap(dirName, size, FilemapProt); ^M
**   addr = PR_MemMap(fm); ^M
**   attr = PR_NewProcessAttr();^M
**   PR_ProcessAttrSetInheritableFileMap( attr, fm, shmname );^M
**   PR_CreateProcess(Client); ^M
**   PR_DestroyProcessAttr(attr);^M
**   ... yadda ...^M
**   PR_MemUnmap( addr );^M
**   PR_CloseFileMap(fm);^M
**^M
**^M
** Client: ^M
**   ... started by server via PR_CreateProcess()^M
**   fm = PR_GetInheritedFileMap( shmname );^M
**   addr = PR_MemMap(fm);^M
**   ... yadda ...^M
**   PR_MemUnmap(addr);^M
**   PR_CloseFileMap(fm);^M
**^M
**^M
** Second Protocol:^M
** Server:^M
**^M
**   fm = PR_OpenAnonFileMap(dirName, size, FilemapProt); ^M
**   fmstring = PR_ExportFileMapAsString( fm );^M
**   addr = PR_MemMap(fm); ^M
**    ... application specific technique to pass fmstring to child^M
**    ... yadda ... Server uses his own magic to create child^M
**   PR_MemUnmap( addr );^M
**   PR_CloseFileMap(fm);^M
**^M
**^M
** Client: ^M
**   ... started by server via his own magic^M
**   ... application specific technique to find fmstring from parent^M
**   fm = PR_ImportFileMapFromString( fmstring )^M
**   addr = PR_MemMap(fm);^M
**   ... yadda ...^M
**   PR_MemUnmap(addr);^M
**   PR_CloseFileMap(fm);^M
**^M
**^M
** lth. 2-Jul-1999.^M
**^M
** Note: The second protocol was requested by NelsonB (7/1999); this is^M
** to accomodate servers which already create their own child processes^M
** using platform native methods.^M
** ^M
*/^M
^M
#ifndef prshma_h___^M
#define prshma_h___^M
^M
#include "prtypes.h"^M
#include "prio.h"^M
#include "prproces.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** PR_OpenAnonFileMap() -- Creates an anonymous file-mapped shared memory^M
**^M
** Description:^M
** PR_OpenAnonFileMap() creates an anonymous shared memory. If the^M
** shared memory already exists, a handle is returned to that shared^M
** memory object.^M
**^M
** On Unix platforms, PR_OpenAnonFileMap() uses 'dirName' as a^M
** directory name, without the trailing '/', to contain the anonymous^M
** file. A filename is generated for the name.^M
**^M
** On Windows platforms, dirName is ignored.^M
**^M
** Inputs:^M
**   dirName -- A directory name to contain the anonymous file.^M
**   size -- The size of the shared memory^M
**   prot -- How the shared memory is mapped. See prio.h^M
**   ^M
** Outputs:^M
**   PRFileMap *^M
**^M
** Returns:^M
**   Pointer to PRFileMap or NULL on error.^M
**^M
*/^M
NSPR_API( PRFileMap *)^M
PR_OpenAnonFileMap(^M
    const char *dirName,^M
    PRSize      size, ^M
    PRFileMapProtect prot^M
);  ^M
^M
/*^M
** PR_ProcessAttrSetInheritableFileMap() -- Prepare FileMap for export  ^M
**   to my children processes via PR_CreateProcess()^M
**^M
** Description:^M
** PR_ProcessAttrSetInheritableFileMap() connects the PRFileMap to^M
** PRProcessAttr with shmname. A subsequent call to PR_CreateProcess()^M
** makes the PRFileMap importable by the child process.^M
**^M
** Inputs:^M
**   attr -- PRProcessAttr, used to pass data to PR_CreateProcess()^M
**   fm -- PRFileMap structure to be passed to the child process^M
**   shmname -- The name for the PRFileMap; used by child.^M
**^M
** Outputs:^M
**   PRFileMap *^M
**^M
** Returns:^M
**   PRStatus^M
**^M
*/^M
NSPR_API(PRStatus) ^M
PR_ProcessAttrSetInheritableFileMap( ^M
    PRProcessAttr   *attr,^M
    PRFileMap       *fm, ^M
    const char      *shmname^M
);^M
^M
/*^M
** PR_GetInheritedFileMap() -- Import a PRFileMap previously exported^M
**   by my parent process via PR_CreateProcess()^M
**^M
** Description:^M
** PR_GetInheritedFileMap() retrieves a PRFileMap object exported from^M
** its parent process via PR_CreateProcess().^M
**^M
** Inputs:^M
**    shmname -- The name provided to PR_ProcessAttrSetInheritableFileMap()^M
** ^M
** Outputs:^M
**   PRFileMap *^M
**^M
** Returns:^M
**   PRFileMap pointer or NULL.^M
**^M
*/^M
NSPR_API( PRFileMap *)^M
PR_GetInheritedFileMap( ^M
    const char *shmname ^M
);^M
^M
/*^M
** PR_ExportFileMapAsString() -- Creates a string identifying a PRFileMap^M
**^M
** Description:^M
** Creates an identifier, as a string, from a PRFileMap object^M
** previously created with PR_OpenAnonFileMap().^M
**^M
** Inputs:^M
**   fm -- PRFileMap pointer to be represented as a string.^M
**   bufsize -- sizeof(buf)^M
**   buf -- a buffer of length PR_FILEMAP_STRING_BUFSIZE^M
**^M
** Outputs:^M
**   buf contains the stringized PRFileMap identifier^M
**^M
** Returns:^M
**   PRStatus^M
**^M
*/^M
NSPR_API( PRStatus )^M
PR_ExportFileMapAsString( ^M
    PRFileMap *fm,^M
    PRSize    bufsize,^M
    char      *buf^M
);^M
#define PR_FILEMAP_STRING_BUFSIZE 128^M
^M
/*^M
** PR_ImportFileMapFromString() -- Creates a PRFileMap from the identifying string^M
**^M
** Description:^M
** PR_ImportFileMapFromString() creates a PRFileMap object from a^M
** string previously created by PR_ExportFileMapAsString().^M
**^M
** Inputs:^M
**   fmstring -- string created by PR_ExportFileMapAsString()^M
**^M
** Returns:^M
**   PRFileMap pointer or NULL.^M
**^M
*/^M
NSPR_API( PRFileMap * )^M
PR_ImportFileMapFromString( ^M
    const char *fmstring^M
);^M
^M
PR_END_EXTERN_C^M
#endif /* prshma_h___ */^M
