/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prtrace_h___^M
#define prtrace_h___^M
/*^M
** prtrace.h -- NSPR's Trace Facility.  		           ^M
**                                                               		           ^M
** The Trace Facility provides a means to trace application				           ^M
** program events within a process. When implementing an         		           ^M
** application program an engineer may insert a "Trace" function 		           ^M
** call, passing arguments to be traced. The "Trace" function     		           ^M
** combines the user trace data with identifying data and        		           ^M
** writes this data in time ordered sequence into a circular     		           ^M
** in-memory buffer; when the buffer fills, it wraps.^M
**                                                               		           ^M
** Functions are provided to set and/or re-configure the size of		           ^M
** the trace buffer, control what events are recorded in the			           ^M
** buffer, enable and disable tracing based on specific user			           ^M
** supplied data and other control functions. Methods are provided		           ^M
** to record the trace entries in the in-memory trace buffer to^M
** a file.^M
**                                                               		           ^M
** Tracing may cause a performance degredation to the application		           ^M
** depending on the number and placement of calls to the tracing		           ^M
** facility. When tracing is compiled in and all tracing is				           ^M
** disabled via the runtime controls, the overhead should be			           ^M
** minimal. ... Famous last words, eh?									           ^M
** 																                   ^M
** When DEBUG is defined at compile time, the Trace Facility is                    ^M
** compiled as part of NSPR and any application using NSPR's                       ^M
** header files will have tracing compiled in. When DEBUG is not                   ^M
** defined, the Trace Facility is not compiled into NSPR nor                       ^M
** exported in its header files.  If the Trace Facility is                         ^M
** desired in a non-debug build, then FORCE_NSPR_TRACE may be                      ^M
** defined at compile time for both the optimized build of NSPR                    ^M
** and the application. NSPR and any application using  NSPR's                     ^M
** Trace Facility must be compiled with the same level of trace                    ^M
** conditioning or unresolved references may be realized at link                   ^M
** time.                                                                           ^M
**                                                                                 ^M
** For any of the Trace Facility methods that requires a trace                     ^M
** handle as an input argument, the caller must ensure that the                    ^M
** trace handle argument is valid. An invalid trace handle                         ^M
** argument may cause unpredictable results.                                       ^M
**                                                                                 ^M
** Trace Facility methods are thread-safe and SMP safe.                            ^M
**                                                                                 ^M
** Users of the Trace Facility should use the defined macros to                     ^M
** invoke trace methods, not the function calls directly. e.g.                      ^M
** PR_TRACE( h1,0,1,2, ...); not PR_Trace(h1,0,1,2, ...);^M
**                                                                                  ^M
** Application designers should be aware of the effects of^M
** debug and optimized build differences when using result of the^M
** Trace Facility macros in expressions.^M
** ^M
** See Also: prcountr.h                                                                                 ^M
**                                                                                  ^M
** /lth. 08-Jun-1998.                                                                                  ^M
*/^M
^M
#include "prtypes.h"^M
#include "prthread.h"^M
#include "prtime.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** Opaque type for the trace handle ^M
** ... Don't even think about looking in here.^M
**^M
*/^M
typedef void *  PRTraceHandle;^M
^M
#if defined (DEBUG) || defined (FORCE_NSPR_TRACE)^M
/*^M
** PRTraceEntry -- A trace entry in the in-memory trace buffer^M
** looks like this.^M
**^M
*/^M
typedef struct PRTraceEntry^M
{^M
    PRThread        *thread;        /* The thread creating the trace entry */^M
    PRTraceHandle   handle;         /* PRTraceHandle creating the trace entry */^M
    PRTime          time;           /* Value of PR_Now() at time of trace entry */^M
    PRUint32        userData[8];    /* user supplied trace data */^M
} PRTraceEntry;^M
^M
/*^M
** PRTraceOption -- command operands to^M
** PR_[Set|Get]TraceOption(). See descriptive meanings there.^M
**^M
*/^M
typedef enum PRTraceOption^M
{^M
    PRTraceBufSize,^M
    PRTraceEnable,              ^M
    PRTraceDisable,^M
    PRTraceSuspend,^M
    PRTraceResume,^M
    PRTraceSuspendRecording,^M
    PRTraceResumeRecording,^M
    PRTraceLockHandles,^M
    PRTraceUnLockHandles,^M
    PRTraceStopRecording^M
} PRTraceOption;^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_DEFINE_TRACE() -- Define a PRTraceHandle^M
** ^M
** DESCRIPTION: PR_DEFINE_TRACE() is used to define a trace^M
** handle.^M
** ^M
*/^M
#define PR_DEFINE_TRACE(name) PRTraceHandle name^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_INIT_TRACE_HANDLE() -- Set the value of a PRTraceHandle^M
** ^M
** DESCRIPTION: ^M
** PR_INIT_TRACE_HANDLE() sets the value of a PRTraceHandle^M
** to value. e.g. PR_INIT_TRACE_HANDLE( myHandle, NULL );^M
** ^M
*/^M
#define PR_INIT_TRACE_HANDLE(handle,value)\^M
    (handle) = (PRCounterHandle)(value)^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_CreateTrace() -- Create a trace handle^M
** ^M
** DESCRIPTION:^M
**  PR_CreateTrace() creates a new trace handle. Tracing is^M
**  enabled for this handle when it is created. The trace handle^M
**  is intended for use in other Trace Facility calls.^M
**  ^M
**  PR_CreateTrace() registers the QName, RName and description^M
**  data so that this data can be retrieved later.^M
** ^M
** INPUTS: ^M
**  qName: pointer to string. QName for this trace handle. ^M
** ^M
**  rName: pointer to string. RName for this trace handle. ^M
** ^M
**  description: pointer to string. Descriptive data about this^M
**  trace handle.^M
**^M
** OUTPUTS:^M
**  Creates the trace handle. ^M
**  Registers the QName and RName with the trace facility.^M
** ^M
** RETURNS: ^M
**  PRTraceHandle^M
** ^M
** RESTRICTIONS:^M
**  qName is limited to 31 characters.^M
**  rName is limited to 31 characters.^M
**  description is limited to 255 characters.^M
** ^M
*/^M
#define PRTRACE_NAME_MAX 31^M
#define PRTRACE_DESC_MAX 255^M
^M
#define PR_CREATE_TRACE(handle,qName,rName,description)\^M
    (handle) = PR_CreateTrace((qName),(rName),(description))^M
^M
NSPR_API(PRTraceHandle)^M
	PR_CreateTrace( ^M
    	const char *qName,          /* QName for this trace handle */^M
	    const char *rName,          /* RName for this trace handle */^M
	    const char *description     /* description for this trace handle */^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_DestroyTrace() -- Destroy a trace handle^M
** ^M
** DESCRIPTION: ^M
**  PR_DestroyTrace() removes the referenced trace handle and^M
** associated QName, RName and description data from the Trace^M
** Facility.^M
** ^M
** INPUTS: handle. A PRTraceHandle^M
** ^M
** OUTPUTS: ^M
**  The trace handle is unregistered.^M
**  The QName, RName and description are removed.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_DESTROY_TRACE(handle)\^M
    PR_DestroyTrace((handle))^M
^M
NSPR_API(void) ^M
	PR_DestroyTrace( ^M
		PRTraceHandle handle    /* Handle to be destroyed */^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_Trace() -- Make a trace entry in the in-memory trace^M
** ^M
** DESCRIPTION:^M
** PR_Trace() makes an entry in the in-memory trace buffer for^M
** the referenced trace handle. The next logically available^M
** PRTraceEntry is used; when the next trace entry would overflow^M
** the trace table, the table wraps.^M
**^M
** PR_Trace() for a specific trace handle may be disabled by^M
** calling PR_SetTraceOption() specifying PRTraceDisable for the^M
** trace handle to be disabled.^M
** ^M
** INPUTS:^M
** handle: PRTraceHandle. The trace handle for this trace.^M
** ^M
** userData[0..7]: unsigned 32bit integers. user supplied data^M
** that is copied into the PRTraceEntry^M
** ^M
** OUTPUTS:^M
**  A PRTraceEntry is (conditionally) formatted in the in-memory^M
** trace buffer.^M
** ^M
** RETURNS: void.^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_TRACE(handle,ud0,ud1,ud2,ud3,ud4,ud5,ud6,ud7)\^M
    PR_Trace((handle),(ud0),(ud1),(ud2),(ud3),(ud4),(ud5),(ud6),(ud7))^M
^M
NSPR_API(void) ^M
	PR_Trace( ^M
    	PRTraceHandle handle,       /* use this trace handle */^M
	    PRUint32    userData0,      /* User supplied data word 0 */^M
	    PRUint32    userData1,      /* User supplied data word 1 */^M
	    PRUint32    userData2,      /* User supplied data word 2 */^M
	    PRUint32    userData3,      /* User supplied data word 3 */^M
	    PRUint32    userData4,      /* User supplied data word 4 */^M
	    PRUint32    userData5,      /* User supplied data word 5 */^M
	    PRUint32    userData6,      /* User supplied data word 6 */^M
	    PRUint32    userData7       /* User supplied data word 7 */^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_SetTraceOption() -- Control the Trace Facility^M
** ^M
** DESCRIPTION:^M
** PR_SetTraceOption() controls the Trace Facility. Depending on^M
** command and value, attributes of the Trace Facility may be^M
** changed.^M
** ^M
** INPUTS:^M
**  command: An enumerated value in the set of PRTraceOption.^M
**  value: pointer to the data to be set. Type of the data is^M
**  dependent on command; for each value of command, the type^M
**  and meaning of dereferenced value is shown.^M
**^M
**  PRTraceBufSize: unsigned long: the size of the trace buffer,^M
** in bytes.^M
** ^M
**  PRTraceEnable: PRTraceHandle. The trace handle to be^M
** enabled.^M
** ^M
**  PRTraceDisable: PRTraceHandle. The trace handle to be^M
** disabled.^M
** ^M
**  PRTraceSuspend: void. value must be NULL. All tracing is^M
** suspended.^M
** ^M
**  PRTraceResume: void. value must be NULL. Tracing for all^M
** previously enabled, prior to a PRTraceSuspend, is resumed.^M
** ^M
**  PRTraceStopRecording: void. value must be NULL. If recording^M
** (see: ** PR_RecordTraceEntries()) is being done, ^M
** PRTraceStopRecording causes PR_RecordTraceEntries() to return^M
** to its caller. If recording is not being done, this function^M
** has no effect.^M
** ^M
**  PRTraceSuspendRecording: void. Must be NULL. If recording is^M
** being done, PRTraceSuspendRecording causes further writes to^M
** the trace file to be suspended. Data in the in-memory^M
** trace buffer that would ordinarily be written to the^M
** trace file will not be written. Trace entries will continue^M
** to be entered in the in-memory buffer. If the Trace Facility^M
** recording is already in a suspended state, the call has no^M
** effect.^M
** ^M
**  PRTraceResumeRecording: void. value must be NULL. If^M
** recording for the Trace Facility has been previously been^M
** suspended, this causes recording to resume. Recording resumes^M
** with the next in-memory buffer segment that would be written^M
** if trace recording had not been suspended. If recording is^M
** not currently suspended, the call has no effect.^M
** ^M
**  PRTraceLockHandles: void. value must be NULL. Locks the^M
** trace handle lock. While the trace handle lock is held,^M
** calls to PR_CreateTrace() will block until the lock is^M
** released.^M
** ^M
**  PRTraceUnlockHandles: void. value must be NULL. Unlocks the^M
** trace handle lock.^M
** ^M
** OUTPUTS:^M
**  The operation of the Trace Facility may be changed.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_SET_TRACE_OPTION(command,value)\^M
    PR_SetTraceOption((command),(value))^M
^M
NSPR_API(void) ^M
	PR_SetTraceOption( ^M
	    PRTraceOption command,  /* One of the enumerated values */^M
	    void *value             /* command value or NULL */^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetTraceOption() -- Retrieve settings from the Trace Facility^M
** ^M
** DESCRIPTION:^M
** PR_GetTraceOption() retrieves the current setting of the^M
** Trace Facility control depending on command.^M
** ^M
** ^M
**  PRTraceBufSize: unsigned long: the size of the trace buffer,^M
** in bytes.^M
** ^M
** ^M
** INPUTS:^M
**  command: one of the enumerated values in PRTraceOptions^M
** valid for PR_GetTraceOption().^M
** ^M
** OUTPUTS:^M
**  dependent on command.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_GET_TRACE_OPTION(command,value)\^M
    PR_GetTraceOption((command),(value))^M
^M
NSPR_API(void) ^M
	PR_GetTraceOption( ^M
    	PRTraceOption command,  /* One of the enumerated values */^M
	    void *value             /* command value or NULL */^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetTraceHandleFromName() -- Retrieve an existing^M
** handle by name.^M
** ^M
** DESCRIPTION:^M
** PR_GetTraceHandleFromName() retreives an existing tracehandle^M
** using the name specified by qName and rName.^M
** ^M
** INPUTS:^M
**  qName: pointer to string. QName for this trace handle. ^M
** ^M
**  rName: pointer to string. RName for this trace handle. ^M
** ^M
** ^M
** OUTPUTS: returned.^M
** ^M
** RETURNS: ^M
**  PRTraceHandle associated with qName and rName or NULL when^M
** there is no match.^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_GET_TRACE_HANDLE_FROM_NAME(handle,qName,rName)\^M
    (handle) = PR_GetTraceHandleFromName((qName),(rName))^M
^M
NSPR_API(PRTraceHandle) ^M
	PR_GetTraceHandleFromName( ^M
    	const char *qName,      /* QName search argument */^M
        const char *rName       /* RName search argument */^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetTraceNameFromHandle() -- Retreive trace name^M
** by bandle.^M
** ^M
** DESCRIPTION:^M
** PR_GetTraceNameFromHandle() retreives the existing qName,^M
** rName, and description for the referenced trace handle.^M
** ^M
** INPUTS: handle: PRTraceHandle.^M
** ^M
** OUTPUTS: pointers to the Trace Facility's copy of qName,^M
** rName and description. ... Don't mess with these values.^M
** They're mine.^M
** ^M
** RETURNS: void^M
** ^M
** RESTRICTIONS:^M
** ^M
*/^M
#define PR_GET_TRACE_NAME_FROM_HANDLE(handle,qName,rName,description)\^M
    PR_GetTraceNameFromHandle((handle),(qName),(rName),(description))^M
^M
NSPR_API(void) ^M
	PR_GetTraceNameFromHandle( ^M
    	PRTraceHandle handle,       /* handle as search argument */^M
	    const char **qName,         /* pointer to associated QName */^M
	    const char **rName,         /* pointer to associated RName */^M
    	const char **description    /* pointer to associated description */^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_FindNextTraceQname() -- Retrieive a QName handle^M
** iterator.^M
** ^M
** DESCRIPTION:^M
** PR_FindNextTraceQname() retreives the first or next trace^M
** QName handle, depending on the value of handle, from the trace^M
** database. The PRTraceHandle returned can be used as an^M
** iterator to traverse the QName handles in the Trace database.^M
** ^M
** INPUTS:^M
**  handle: When NULL, PR_FindNextQname() returns the first QName^M
** handle. When a handle is a valid PRTraceHandle previously^M
** retreived using PR_FindNextQname() the next QName handle is^M
** retreived.^M
** ^M
** OUTPUTS: returned.^M
** ^M
** RETURNS: ^M
**  PRTraceHandle or NULL when there are no trace handles.^M
** ^M
** RESTRICTIONS:^M
**  Iterating thru the trace handles via FindFirst/FindNext^M
** should be done under protection of the trace handle lock.^M
** See: PR_SetTraceOption( PRLockTraceHandles ).^M
** ^M
*/^M
#define PR_FIND_NEXT_TRACE_QNAME(next,handle)\^M
    (next) = PR_FindNextTraceQname((handle))^M
^M
NSPR_API(PRTraceHandle) ^M
	PR_FindNextTraceQname( ^M
        PRTraceHandle handle^M
);^M
^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_FindNextTraceRname() -- Retrieive an RName handle^M
** iterator.^M
** ^M
** DESCRIPTION:^M
** PR_FindNextTraceRname() retreives the first or next trace^M
** RName handle, depending on the value of handle, from the trace^M
** database. The PRTraceHandle returned can be used as an^M
** iterator to traverse the RName handles in the Trace database.^M
** ^M
** INPUTS:^M
**  rhandle: When NULL, PR_FindNextRname() returns the first^M
** RName handle. When a handle is a valid PRTraceHandle^M
** previously retreived using PR_FindNextRname() the next RName^M
** handle is retreived.^M
**  qhandle: A valid PRTraceHandle retruned from a previous call^M
** to PR_FIND_NEXT_TRACE_QNAME().^M
** ^M
** OUTPUTS: returned.^M
** ^M
** RETURNS: ^M
**  PRTraceHandle or NULL when there are no trace handles.^M
** ^M
** RESTRICTIONS:^M
**  Iterating thru the trace handles via FindNext should be done^M
** under protection of the trace handle lock. See: (^M
** PR_SetTraceOption( PRLockTraceHandles ).^M
** ^M
*/^M
#define PR_FIND_NEXT_TRACE_RNAME(next,rhandle,qhandle)\^M
    (next) = PR_FindNextTraceRname((rhandle),(qhandle))^M
^M
NSPR_API(PRTraceHandle) ^M
	PR_FindNextTraceRname( ^M
        PRTraceHandle rhandle,^M
        PRTraceHandle qhandle^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_RecordTraceEntries() -- Write trace entries to external media^M
** ^M
** DESCRIPTION:^M
** PR_RecordTraceEntries() causes entries in the in-memory trace^M
** buffer to be written to external media.^M
**^M
** When PR_RecordTraceEntries() is called from an application^M
** thread, the function appears to block until another thread^M
** calls PR_SetTraceOption() with the PRTraceStopRecording^M
** option. This suggests that PR_RecordTraceEntries() should be^M
** called from a user supplied thread whose only job is to^M
** record trace entries. ^M
** ^M
** The environment variable NSPR_TRACE_LOG controls the operation^M
** of this function. When NSPR_TRACE_LOG is not defined in the^M
** environment, no recording of trace entries occurs. When^M
** NSPR_TRACE_LOG is defined, the value of its definition must be^M
** the filename of the file to receive the trace entry buffer.^M
**^M
** PR_RecordTraceEntries() attempts to record the in-memory^M
** buffer to a file, subject to the setting of the environment^M
** variable NSPR_TRACE_LOG. It is possible because of system^M
** load, the thread priority of the recording thread, number of^M
** active trace records being written over time, and other^M
** variables that some trace records can be lost. ... In other^M
** words: don't bet the farm on getting everything.^M
** ^M
** INPUTS: none^M
** ^M
** OUTPUTS: none^M
** ^M
** RETURNS: PR_STATUS^M
**    PR_SUCCESS no errors were found.^M
**    PR_FAILURE errors were found.^M
** ^M
** RESTRICTIONS:^M
** Only one thread can call PR_RecordTraceEntries() within a^M
** process.^M
** ^M
** On error, PR_RecordTraceEntries() may return prematurely.^M
** ^M
*/^M
#define PR_RECORD_TRACE_ENTRIES()\^M
	PR_RecordTraceEntries()^M
    ^M
NSPR_API(void)^M
	PR_RecordTraceEntries(^M
        void ^M
);^M
^M
/* -----------------------------------------------------------------------^M
** FUNCTION: PR_GetTraceEntries() -- Retreive trace entries from^M
** the Trace Facility^M
** ^M
** DESCRIPTION:^M
** PR_GetTraceEntries() retreives trace entries from the Trace^M
** Facility. Up to count trace entries are copied from the Trace^M
** Facility into buffer. Only those trace entries that have not^M
** been copied via a previous call to PR_GetTraceEntries() are^M
** copied. The actual number copied is placed in the PRInt32^M
** variable pointed to by found.^M
**^M
** If more than count trace entries have entered the Trace^M
** Facility since the last call to PR_GetTraceEntries() ^M
** a lost data condition is returned. In this case, the most^M
** recent count trace entries are copied into buffer and found is^M
** set to count.^M
** ^M
** INPUTS:^M
**  count. The number of trace entries to be copied into buffer.^M
** ^M
** ^M
** OUTPUTS:^M
**  buffer. An array of PRTraceEntries. The buffer is supplied^M
** by the caller.^M
** ^M
** found: 32bit signed integer. The number of PRTraceEntries^M
** actually copied. found is always less than or equal to count.^M
** ^M
** RETURNS: ^M
**  zero when there is no lost data.^M
**  non-zero when some PRTraceEntries have been lost.^M
** ^M
** RESTRICTIONS:^M
** This is a real performance pig. The copy out operation is bad^M
** enough, but depending on then frequency of calls to the^M
** function, serious performance impact to the operating^M
** application may be realized. ... YMMV.^M
** ^M
*/^M
#define PR_GET_TRACE_ENTRIES(buffer,count,found)\^M
        PR_GetTraceEntries((buffer),(count),(found))^M
^M
^M
NSPR_API(PRIntn)^M
    PR_GetTraceEntries(^M
        PRTraceEntry    *buffer,    /* where to write output */^M
        PRInt32         count,      /* number to get */^M
        PRInt32         *found      /* number you got */^M
);^M
^M
#else /* !(defined (DEBUG) || defined (FORCE_NSPR_TRACE)) */^M
/*^M
** Define the Trace Facility macros as No-Ops for when the trace^M
** facility is to be compiled-out of the application.^M
**^M
*/^M
#define PR_DEFINE_TRACE(name) PRTraceHandle name^M
#define PR_INIT_TRACE_HANDLE(handle,value)^M
#define PR_CREATE_TRACE(handle,qName,rName,description)^M
#define PR_DESTROY_TRACE(handle)^M
#define PR_TRACE(handle,ud0,ud1,ud2,ud3,ud4,ud5,ud6,ud7)^M
#define PR_SET_TRACE_OPTION(command,value)^M
#define PR_GET_TRACE_OPTION(command,value)^M
#define PR_GET_TRACE_HANDLE_FROM_NAME(handle,qName,rName)^M
#define PR_GET_TRACE_NAME_FROM_HANDLE(handle,qName,rName,description)^M
#define PR_FIND_NEXT_TRACE_QNAME(next,handle)^M
#define PR_FIND_NEXT_TRACE_RNAME(next,rhandle,qhandle)^M
#define PR_GET_TRACE_ENTRIES(buffer,count,found)^M
#define PR_RECORD_TRACE_ENTRIES()^M
^M
#endif /* !(defined (DEBUG) || defined (FORCE_NSPR_TRACE)) */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prtrace_h___ */^M
^M
